@@ -1,11 +1,12 @@
 /*
  * fs-traverse.c
- * $Id: fs-traverse.c 51280 2009-05-22 06:27:45Z toby@macports.org $
+ * $Id: fs-traverse.c 65785 2010-03-31 22:25:04Z raimue@macports.org $
  *
  * Find files and execute arbitrary expressions on them.
- * Author: Jordan K. Hubbard, Kevin Ballard
+ * Author: Jordan K. Hubbard, Kevin Ballard, Rainer Mueller
  *
  * Copyright (c) 2004 Apple Computer, Inc.
+ * Copyright (c) 2010 The MacPorts Project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,25 +38,15 @@
 #include <config.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <errno.h>
+#include <fts.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#if HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
-#include <fts.h>
-#include <errno.h>
-
-#if HAVE_LIMITS_H
-#include <limits.h>
-#endif
-
 #include <tcl.h>
 
 #include "fs-traverse.h"
@@ -64,8 +55,9 @@ static int do_traverse(Tcl_Interp *interp, int flags, char * CONST *targets, Tcl
 
 #define F_DEPTH 0x1
 #define F_IGNORE_ERRORS 0x2
+#define F_TAILS 0x4
 
-/* fs-traverse ?-depth? ?-ignoreErrors? ?--? varname target-list body */
+/* fs-traverse ?-depth? ?-ignoreErrors? ?-tails? ?--? varname target-list body */
 int
 FsTraverseCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
@@ -94,6 +86,11 @@ FsTraverseCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Ob
             ++objv, --objc;
             continue;
         }
+        if (!strcmp(arg, "-tails")) {
+            flags |= F_TAILS;
+            ++objv, --objc;
+            continue;
+        }
         if (!strcmp(arg, "--")) {
             ++objv, --objc;
             break;
@@ -103,7 +100,7 @@ FsTraverseCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Ob
     
     /* Parse remaining args */
     if (objc != 3) {
-        Tcl_WrongNumArgs(interp, 1, objv_orig, "?-depth? ?-ignoreErrors? ?--? varname target-list body");
+        Tcl_WrongNumArgs(interp, 1, objv_orig, "?-depth? ?-ignoreErrors? ?-tails? ?--? varname target-list body");
         return TCL_ERROR;
     }
     
@@ -116,8 +113,17 @@ FsTraverseCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Ob
     body = *objv;
     
     if ((rval = Tcl_ListObjGetElements(interp, listPtr, &lobjc, &lobjv)) == TCL_OK) {
-        char **entries = calloc(lobjc+1, sizeof(char *));
-        char **iter = (char **)entries;
+        char **entries;
+        char **iter;
+
+        if (flags & F_TAILS && lobjc > 1) {
+            /* result would be ambiguous with multiple paths, so we do not allow this */
+            Tcl_SetResult(interp, "-tails cannot be used with multiple paths", TCL_STATIC);
+            return TCL_ERROR;
+        }
+
+        entries = calloc(lobjc+1, sizeof(char *));
+        iter = (char **)entries;
         while (lobjc > 0) {
             *iter++ = Tcl_GetString(*lobjv);
             --lobjc, ++lobjv;
@@ -129,6 +135,23 @@ FsTraverseCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Ob
     return rval;
 }
 
+static const char *
+extract_tail(const char *target, const char *path)
+{
+    const char *xpath = path;
+    size_t tlen = strlen(target);
+
+    if (strncmp(xpath, target, tlen) == 0) {
+        if (*(xpath + tlen) == '\0') {
+            xpath = ".";
+        } else if (*(xpath + tlen) == '/') {
+            xpath += tlen + 1;
+        }
+    }
+
+    return xpath;
+}
+
 static int
 do_compare(const FTSENT **a, const FTSENT **b)
 {
@@ -150,7 +173,14 @@ do_traverse(Tcl_Interp *interp, int flags, char * CONST *targets, Tcl_Obj *varna
             case FTS_DP: /* directory in post-order*/
             {
                 if (!(flags & F_DEPTH) != !(ent->fts_info == FTS_D)) {
-                    Tcl_Obj *rpath, *path = Tcl_NewStringObj(ent->fts_path, ent->fts_pathlen);
+                    Tcl_Obj *rpath, *path;
+                    if (flags & F_TAILS) {
+                        /* there cannot be multiple targets */
+                        const char *xpath = extract_tail(targets[0], ent->fts_path);
+                        path = Tcl_NewStringObj(xpath, -1);
+                    } else {
+                        path = Tcl_NewStringObj(ent->fts_path, ent->fts_pathlen);
+                    }
                     Tcl_IncrRefCount(path);
                     rpath = Tcl_ObjSetVar2(interp, varname, NULL, path, TCL_LEAVE_ERR_MSG);
                     Tcl_DecrRefCount(path);
@@ -175,7 +205,14 @@ do_traverse(Tcl_Interp *interp, int flags, char * CONST *targets, Tcl_Obj *varna
             case FTS_SLNONE: /* symbolic link with non-existant target */
             case FTS_DEFAULT: /* file type not otherwise handled (e.g., fifo) */
             {
-                Tcl_Obj *rpath, *path = Tcl_NewStringObj(ent->fts_path, ent->fts_pathlen);
+                Tcl_Obj *rpath, *path;
+                if (flags & F_TAILS) {
+                    /* there cannot be multiple targets */
+                    const char *xpath = extract_tail(targets[0], ent->fts_path);
+                    path = Tcl_NewStringObj(xpath, -1);
+                } else {
+                    path = Tcl_NewStringObj(ent->fts_path, ent->fts_pathlen);
+                }
                 Tcl_IncrRefCount(path);
                 rpath = Tcl_ObjSetVar2(interp, varname, NULL, path, TCL_LEAVE_ERR_MSG);
                 Tcl_DecrRefCount(path);
@@ -201,7 +238,8 @@ do_traverse(Tcl_Interp *interp, int flags, char * CONST *targets, Tcl_Obj *varna
             {
                 if (!(flags & F_IGNORE_ERRORS)) {
                     Tcl_SetErrno(ent->fts_errno);
-                    Tcl_SetResult(interp, (char *)Tcl_PosixError(interp),  TCL_STATIC);
+                    Tcl_ResetResult(interp);
+                    Tcl_AppendResult(interp, ent->fts_path, ": ", (char *)Tcl_PosixError(interp), NULL); 
                     fts_close(root_fts);
                     return TCL_ERROR;
                 }
@@ -210,6 +248,7 @@ do_traverse(Tcl_Interp *interp, int flags, char * CONST *targets, Tcl_Obj *varna
     }
     /* check errno before calling fts_close in case it sets errno to 0 on success */
     if (errno != 0 || (fts_close(root_fts) != 0 && !(flags & F_IGNORE_ERRORS))) {
+        Tcl_SetErrno(errno);
         Tcl_SetResult(interp, (char *)Tcl_PosixError(interp), TCL_STATIC);
         return TCL_ERROR;
     }
