@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portfetch.tcl
-# $Id: portfetch.tcl 68818 2010-06-14 17:09:28Z jmr@macports.org $
+# $Id: portfetch.tcl 72839 2010-10-28 17:56:39Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Computer, Inc.
 # All rights reserved.
@@ -148,7 +148,7 @@ proc portfetch::set_extract_type {option action args} {
             }
             use_xz {
                 set extract.suffix .tar.xz
-                depends_extract-append bin:xz:xz-devel
+                depends_extract-append bin:xz:xz
             }
             use_zip {
                 set extract.suffix .zip
@@ -460,18 +460,26 @@ proc portfetch::fetchfiles {args} {
     set sorted no
 
     foreach {url_var distfile} $fetch_urls {
-        if {![file isfile $distpath/$distfile] && ($usealtworkpath || ![file isfile ${altprefix}${distpath}/$distfile])} {
+        if {![file isfile "${distpath}/${distfile}"]} {
             ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $distfile $distpath]"
             if {![file writable $distpath]} {
                 return -code error [format [msgcat::mc "%s must be writable"] $distpath]
             }
+            if {!$usealtworkpath && [file isfile ${altprefix}${distpath}/${distfile}]} {
+                if {[catch {file link -hard "${distpath}/${distfile}" "${altprefix}${distpath}/${distfile}"}]} {
+                    ui_debug "failed to hardlink ${distfile} into distpath, copying instead"
+                    file copy "${altprefix}${distpath}/${distfile}" "${distpath}/${distfile}"
+                }
+                ui_info "Found $distfile in ${altprefix}${distpath}"
+                continue
+            }
             if {!$sorted} {
                 sortsites fetch_urls [mirror_sites $fallback_mirror_site {} {} [get_full_mirror_sites_path]] master_sites
                 set sorted yes
             }
             if {![info exists urlmap($url_var)]} {
                 ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
-                set urlmap($url_var) $master_sites
+                set urlmap($url_var) $urlmap(master_sites)
             }
             unset -nocomplain fetched
             foreach site $urlmap($url_var) {
